==> karlender-v0.9.1.tar.gz: karlender-v0.9.1/src/widgets/dumb/infinity_carousel.rs <==
use gdk4::subclass::prelude::{ObjectImpl, ObjectSubclassIsExt};
use gtk::{prelude::*, TemplateChild};
use libadwaita as adw;
use std::cell::Cell;

type Creator = Box<dyn Fn(i64) -> gtk::Widget + 'static>;

#[widget(extends gtk::Box)]
#[template(file = "infinity_carousel.ui")]
struct InfinityCarousel {
    #[property_i64]
    pub current: Cell<i64>,

    pub item: Cell<Option<Creator>>,

    pub carousel_mode: Cell<bool>,

    #[template_child]
    pub carousel: TemplateChild<adw::Carousel>,

    #[template_child]
    pub immediate_box: TemplateChild<gtk::Box>,

    #[template_child]
    pub next_button: TemplateChild<gtk::Button>,
    #[template_child]
    pub prev_button: TemplateChild<gtk::Button>,

    #[signal_handler(next_button clicked)]
    on_next: (),
    #[signal_handler(prev_button clicked)]
    on_prev: (),

    #[signal]
    pub page_changed: (),
}

impl InfinityCarousel {
    pub fn new(creator: impl Fn(i64) -> gtk::Widget + 'static) -> Self {
        let carousel: Self =
            glib::Object::new(&[("current", &0i64)]).expect("Failed to create month page");
        carousel.imp().item.set(Some(Box::new(creator)));
        Self::constructed(&carousel);
        carousel
    }

    pub fn constructed(&self) {
        if let Some(item) = self.imp().item.take() {
            self.imp().item.set(Some(item));

            self.imp().carousel_mode.set(false);
            self.immediate_mode();
        }
    }

    pub fn on_next(&self, _: gtk::Button) {
        let current = self.imp().current.take();
        if let Some(c) = self.immediate_box().first_child() {
            self.immediate_box().remove(&c);
        }
        self.imp().current.set(current + 1);
        let item = self.imp().item.take().unwrap();
        self.immediate_box().append(&item(current + 1));
        self.imp().item.set(Some(item));
        self.emit_page_changed();
    }

    pub fn on_prev(&self, _: gtk::Button) {
        let current = self.imp().current.take();
        if let Some(c) = self.immediate_box().first_child() {
            self.immediate_box().remove(&c);
        }
        self.imp().current.set(current - 1);
        let item = self.imp().item.take().unwrap();
        self.immediate_box().append(&item(current - 1));
        self.imp().item.set(Some(item));
        self.emit_page_changed();
    }

    fn immediate_mode(&self) {
        let item = self.imp().item.take().unwrap();
        self.imp().current.set(0);
        self.immediate_box().append(&item(0));
        self.imp().item.set(Some(item));
        self.emit_page_changed();

        let _self = self;
        glib::MainContext::default().spawn_local(glib::clone!(@weak _self => async move {
            _self.emit_page_changed()
        }));
    }

    pub fn current_page<T: IsA<gtk::Widget>>(&self) -> Option<T> {
        let carousel_mode = self.imp().carousel_mode.clone().take();

        if carousel_mode {
            if self.carousel().n_pages() == 1 {
                self.carousel().nth_page(0).downcast().ok()
            } else {
                let nth = self.imp().current.clone().take();
                self.carousel().nth_page(nth as u32).downcast().ok()
            }
        } else {
            self.immediate_box().first_child().unwrap().downcast().ok()
        }
    }

    pub fn connect_page_changed(&self, f: impl Fn(&Self) + 'static) {
        self._connect_page_changed(f);
    }

    fn _on_page_changed(&self, _: u32) {
        self.emit_page_changed();
    }

    pub fn set_current(&self, current: i64) {
        let _self = self;
        glib::MainContext::default().spawn_local(glib::clone!(@weak _self => async move {
            _self.imp().current.set(current);
            if let Some(c) = _self.immediate_box().first_child() {
                _self.immediate_box().remove(&c);
            }
            if let Some(item) = _self.imp().item.take() {
                _self.immediate_box().append(&item(current));
                _self.imp().item.set(Some(item));
            }
            _self.emit_page_changed();
        }));
    }

    pub fn show_nav_buttons(&self, show: bool) {
        self.prev_button().set_visible(show);
        self.next_button().set_visible(show);
    }
}

==> karlender-v0.9.2.tar.gz: karlender-v0.9.2/src/widgets/dumb/infinity_carousel.rs <==
use gdk4::subclass::prelude::{ObjectImpl, ObjectSubclassIsExt};
use gtk::{prelude::*, TemplateChild};
use libadwaita as adw;
use std::cell::Cell;

type Creator = Box<dyn Fn(i64) -> gtk::Widget + 'static>;

#[widget(extends gtk::Box)]
#[template(file = "infinity_carousel.ui")]
struct InfinityCarousel {
    #[property_i64]
    pub current: Cell<i64>,

    pub item: Cell<Option<Creator>>,

    pub carousel_mode: Cell<bool>,

    #[template_child]
    pub carousel: TemplateChild<adw::Carousel>,

    #[template_child]
    pub immediate_box: TemplateChild<gtk::Box>,

    #[template_child]
    pub next_button: TemplateChild<gtk::Button>,
    #[template_child]
    pub prev_button: TemplateChild<gtk::Button>,

    #[signal_handler(next_button clicked)]
    on_next: (),
    #[signal_handler(prev_button clicked)]
    on_prev: (),

    #[signal]
    pub page_changed: (),
}

impl InfinityCarousel {
    pub fn new(creator: impl Fn(i64) -> gtk::Widget + 'static) -> Self {
        let carousel: Self = glib::Object::new();
        carousel.set_properties(&[("current", &0i64)]);
        carousel.imp().item.set(Some(Box::new(creator)));
        Self::constructed(&carousel);
        carousel
    }

    pub fn constructed(&self) {
        if let Some(item) = self.imp().item.take() {
            self.imp().item.set(Some(item));

            self.imp().carousel_mode.set(false);
            self.immediate_mode();
        }
    }

    pub fn on_next(&self, _: gtk::Button) {
        let current = self.imp().current.take();
        if let Some(c) = self.immediate_box().first_child() {
            self.immediate_box().remove(&c);
        }
        self.imp().current.set(current + 1);
        let item = self.imp().item.take().unwrap();
        self.immediate_box().append(&item(current + 1));
        self.imp().item.set(Some(item));
        self.emit_page_changed();
    }

    pub fn on_prev(&self, _: gtk::Button) {
        let current = self.imp().current.take();
        if let Some(c) = self.immediate_box().first_child() {
            self.immediate_box().remove(&c);
        }
        self.imp().current.set(current - 1);
        let item = self.imp().item.take().unwrap();
        self.immediate_box().append(&item(current - 1));
        self.imp().item.set(Some(item));
        self.emit_page_changed();
    }

    fn immediate_mode(&self) {
        let item = self.imp().item.take().unwrap();
        self.imp().current.set(0);
        self.immediate_box().append(&item(0));
        self.imp().item.set(Some(item));
        self.emit_page_changed();

        let _self = self;
        glib::MainContext::default().spawn_local(glib::clone!(@weak _self => async move {
            _self.emit_page_changed()
        }));
    }

    pub fn current_page<T: IsA<gtk::Widget>>(&self) -> Option<T> {
        let carousel_mode = self.imp().carousel_mode.clone().take();

        if carousel_mode {
            if self.carousel().n_pages() == 1 {
                self.carousel().nth_page(0).downcast().ok()
            } else {
                let nth = self.imp().current.clone().take();
                self.carousel().nth_page(nth as u32).downcast().ok()
            }
        } else {
            self.immediate_box().first_child().unwrap().downcast().ok()
        }
    }

    pub fn connect_page_changed(&self, f: impl Fn(&Self) + 'static) {
        self._connect_page_changed(f);
    }

    fn _on_page_changed(&self, _: u32) {
        self.emit_page_changed();
    }

    pub fn set_current(&self, current: i64) {
        let _self = self;
        glib::MainContext::default().spawn_local(glib::clone!(@weak _self => async move {
            _self.imp().current.set(current);
            if let Some(c) = _self.immediate_box().first_child() {
                _self.immediate_box().remove(&c);
            }
            if let Some(item) = _self.imp().item.take() {
                _self.immediate_box().append(&item(current));
                _self.imp().item.set(Some(item));
            }
            _self.emit_page_changed();
        }));
    }

    pub fn show_nav_buttons(&self, show: bool) {
        self.prev_button().set_visible(show);
        self.next_button().set_visible(show);
    }
}
